#include <RH_ASK.h>                            // Include the Radio Head library
#include <LiquidCrystal.h>                     // Include the LCD Library
LiquidCrystal lcd(5, 6, 10, 11, 12, 13);       // Atmega pin 5 = RS, Pin 6 = Enable, Pins 10 - 13 = D4 to D7  // Set up LCD pins
RH_ASK driver(2000, 4, 12);                    // Set the Radio Head transmit parameters to 2 kbps, Rx on pin 4, Tx on pin 12

long DisplayTime;                              // Var used to keep track of when to display battery voltage
int FlagC = 1;                                 // Flag used to indicate if LCD display should be cleared


struct dataStruct{                             // Create stucture for variables to be received     
  float TxCent;                                // Temperature in deg C to be received 
  float TxFarh;                                // Temperature in deg F to be received 
  float BattVolts;                             // Transmitter circuit battery voltage to be received 
} myData;

void setup() {
    
    lcd.begin(16,2);                           // Initialize for a 16 x 2 LCD
    lcd.clear();                               // Clear LCD display
    Serial.begin(9600);                        // Start communication with the serial monitor only as required
    if (!driver.init())                        // Check if Radio Head properly configured 
    Serial.println("RH initiation failure");

    DisplayTime = millis() + 60000;            // Initialize DisplayTime value to 1 minute (i.e Display battery voltage every min)
    lcd.setCursor(0,0);                        // Set cursor to first position, first line
    lcd.print("Wait for Data...");             // Display "Wait for Data" Msg
    
}

void loop(){                                  // The receiver program
    uint8_t buf[RH_ASK_MAX_MESSAGE_LEN];      // Determine the length of msg
    uint8_t buflen = sizeof(buf);             // Set buffer to required length

    if (driver.recv(buf, &buflen)) {          // Message comming in, not blocking, checksum ok
  
        memcpy(&myData, buf, sizeof(myData)); // Get data, data size
        
        Serial.println("");                   // Print a space
       
        Serial.print("Temperature Deg C: ");  
        Serial.println(myData.TxCent);        // Print the received deg C temperature value
       
        Serial.print("Temperature Deg F: ");
        Serial.println(myData.TxFarh);        // Print the received deg F temperature value

        Serial.print("Battery Volts:      ");
        Serial.println(myData.BattVolts);     // Print the received Tx circuit battery level
        LcdDisplay();                         // Go to the LCD display sub
    }
    
    if(millis() == 180000) {                  // If microcontroller clock has been running for 3 minutes
}
 }

void LcdDisplay()                             // This sub displays the received values on the LCD
{
  if (FlagC == 1) {                           // When FlagC = 1 clear LCD display first time only
  lcd.clear();                                // Clear LCD display
  }
  FlagC = 0;                                  // Set FlagC (clear LCD display flag) to zero i.e don't clear repeatedly
  lcd.setCursor(0,0);                         // Set cursor to first position, first line
  lcd.print("Deg C:");                        // Display "Deg C"             
  lcd.setCursor(7,0);                         // Set cursor to seventh position, first line
  lcd.print(myData.TxCent);                   // Display the received deg C temperature value
  lcd.setCursor(13,0);                        // Set cursor to thirteenth position, first line
  lcd.print(char(223));                       // Display the "degrees" symbol
  lcd.print("C");                             // Display the "C" 
  
  lcd.setCursor(0,1);                         // Set cursor to first position, second line
  lcd.print("Deg F:");                        // Display "Deg F"
  lcd.setCursor(7,1);                         // Set cursor to seventh position, second line                   
  lcd.print(myData.TxFarh);                   // Display the received deg F temperature value
  lcd.setCursor(13,1);                        // Set cursor to thirteenth position, second line
  lcd.print(char(223));                       // Display the "degrees" symbol                              
  lcd.print("F");                             // Display the "F" 
  lcd.setCursor(15,0);                        // Set cursor to last position, first line
  lcd.print("*");                             // Display the "*" symbol to indicate data received
  delay(500);                                 // Wait 500 msec
  lcd.setCursor(15,0);                        // Set cursor to last position, first line
  lcd.print(" ");                             // Blank out the "*" symbol
  delay(500);                                 // Wait 500 msec
  
  if (millis() > DisplayTime) {               // If current run time is more than DisplayTime
  lcd.clear();                                // Clear the LCD
  lcd.setCursor(0,0);                         // Set cursor to first position, first line
  lcd.print("Tx Battery Volts");              // Display "Tx Battery Volts"
  lcd.setCursor(0,1);                         // Set cursor to first position, second line
  lcd.print(myData.BattVolts);                // Display the received battery voltage value
  lcd.setCursor(4,1);                         // Set cursor to fourth position, second line
  lcd.print("V");                             // Display "V"
  delay(3000);                                // Wait for 3 seconds while displaying
  lcd.clear();                                // Clear the LCD
  
                                              // Display last known temperature values
  lcd.setCursor(0,0);                         // Set cursor to first position, first line
  lcd.print("Deg C:");                        // Display "Deg C"
  lcd.setCursor(7,0);                         // Set cursor to seventh position, first line
  lcd.print(myData.TxCent);                   // Display the received deg C temperature value
  lcd.setCursor(13,0);                        // Set cursor to thirteenth position, first line
  lcd.print(char(223));                       // Display the "degrees" symbol
  
  lcd.setCursor(0,1);                         // Set cursor to first position, second line
  lcd.print("Deg F:");                        // Display "Deg F"
  lcd.setCursor(7,1);                         // Set cursor to seventh position, second line
  lcd.print(myData.TxFarh);                   // Display the received deg F temperature value
  lcd.setCursor(13,1);                        // Set cursor to thirteenth position, second line
  lcd.print(char(223));                       // Display the "degrees" symbol                              
                               
  DisplayTime = millis() + 60000;             // Increment the DisplayTime var
  
  }
   }
    
